/*!
 * Copyright (c) 2015-2019 Cisco Systems, Inc. See LICENSE file.
 */

import {assert} from '@webex/test-helper-chai';
import sinon from 'sinon';
import Presence from '@webex/internal-plugin-presence';
import MockWebex from '@webex/test-helper-mock-webex';

describe('plugin-presence', () => {
  describe('Presence', () => {
    let webex;

    beforeEach(() => {
      webex = new MockWebex({
        children: {
          presence: Presence
        }
      });
    });

    describe('#get()', () => {
      it('requires a person id parameter', () => assert.isRejected(webex.internal.presence.get(), /A person id is required/));
    });

    describe('#list()', () => {
      it('requires a parameter', () => assert.isRejected(webex.internal.presence.list(), /An array of person ids is required/));

      it('requires a person array parameter', () => assert.isRejected(webex.internal.presence.list('abc'), /An array of person ids is required/));
    });

    describe('#subscribe()', () => {
      it('requires a person parameter', () => assert.isRejected(webex.internal.presence.subscribe(), /A person id is required/));
      it('subscription request called twice with batch of 100 ids', () => {
        const ids = [...Array(100).keys()];

        webex.request = function (options) {
          return Promise.resolve({
            statusCode: 204,
            body: [],
            options
          });
        };
        sinon.spy(webex, 'request');

        webex.internal.presence.subscribe(ids);
        assert.calledTwice(webex.request);
      });
    });

    describe('#unsubscribe()', () => {
      it('requires a person parameter', () => assert.isRejected(webex.internal.presence.unsubscribe(), /A person id is required/));
    });

    describe('#setStatus()', () => {
      it('requires a status', () => assert.isRejected(webex.internal.presence.setStatus(), /A status is required/));
    });
  });
});
