'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _weakMap = require('babel-runtime/core-js/weak-map');

var _weakMap2 = _interopRequireDefault(_weakMap);

var _common = require('@webex/common');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var usersByEmail = new _weakMap2.default(); /*!
                                             * Copyright (c) 2015-2019 Cisco Systems, Inc. See LICENSE file.
                                             */

var usersById = new _weakMap2.default();

/**
 * @class
 */

var UserUUIDStore = function () {
  /**
   * @constructs {UserUUIDStore}
   */
  function UserUUIDStore() {
    (0, _classCallCheck3.default)(this, UserUUIDStore);

    usersByEmail.set(this, new _map2.default());
    usersById.set(this, new _map2.default());
  }

  /**
   * Adds a user object to the store
   * @param {Object} user
   * @param {string} user.id
   * @param {string} user.emailAddress
   * @returns {Promise}
   */


  (0, _createClass3.default)(UserUUIDStore, [{
    key: 'add',
    value: function add(user) {
      var _this = this;

      if (!user.id) {
        return _promise2.default.reject(new Error('`user.id` is required'));
      }

      if (!user.emailAddress) {
        return _promise2.default.reject(new Error('`user.emailAddress` is required'));
      }

      if (!_common.patterns.uuid.test(user.id)) {
        return _promise2.default.reject(new Error('`user.id` does not appear to be a uuid'));
      }

      if (!_common.patterns.email.test(user.emailAddress)) {
        return _promise2.default.reject(new Error('`user.emailAddress` does not appear to be an email address'));
      }

      var p1 = this.getById(user.id).then(function (u) {
        return usersById.get(_this).set(user.id, (0, _assign2.default)({}, u, user));
      }).catch(function () {
        return usersById.get(_this).set(user.id, (0, _assign2.default)({}, user));
      });

      var p2 = this.getByEmail(user.emailAddress).then(function (u) {
        return usersByEmail.get(_this).set(user.emailAddress, (0, _assign2.default)({}, u, user));
      }).catch(function () {
        return usersByEmail.get(_this).set(user.emailAddress, (0, _assign2.default)({}, user));
      });

      return _promise2.default.all([p1, p2]);
    }

    /**
     * Retrievves the specified user object from the store
     * @param {string} id
     * @returns {Promise<Object>}
     */

  }, {
    key: 'get',
    value: function get(id) {
      if (_common.patterns.uuid.test(id)) {
        return this.getById(id);
      }

      if (_common.patterns.email.test(id)) {
        return this.getByEmail(id);
      }

      return _promise2.default.reject(new Error('`id` does not appear to be a valid user identifier'));
    }

    /**
     * Retrieves the specified user object by id from the store
     * @param {Object} id
     * @returns {Promise<Object>}
     */

  }, {
    key: 'getById',
    value: function getById(id) {
      var ret = usersById.get(this).get(id);

      if (ret) {
        return _promise2.default.resolve(ret);
      }

      return _promise2.default.reject(new Error('No user found by specified id'));
    }

    /**
     * Retrieves the specified user object by id from the store
     * @param {Object} email
     * @returns {Promise<Object>}
     */

  }, {
    key: 'getByEmail',
    value: function getByEmail(email) {
      var ret = usersByEmail.get(this).get(email);

      if (ret) {
        return _promise2.default.resolve(ret);
      }

      return _promise2.default.reject(new Error('No user found by specified email address'));
    }
  }]);
  return UserUUIDStore;
}();

exports.default = UserUUIDStore;
//# sourceMappingURL=user-uuid-store.js.map
