'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _isArray2 = require('lodash/isArray');

var _isArray3 = _interopRequireDefault(_isArray2);

var _isFinite2 = require('lodash/isFinite');

var _isFinite3 = _interopRequireDefault(_isFinite2);

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _constants = require('../constants');

var _parameter = require('../common/errors/parameter');

var _parameter2 = _interopRequireDefault(_parameter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var StatsAnalyzer = {};

/**
 * Can involve changing of the default plugin-meetings sdk for deeper results
 * @param {Array} series of WebRTCData
 * @param {Object} options
 * @param {Array} options.analysisKeys [{key: 'bytesSent', check: 'increasing'}, {key: 'bytesReceived', check: 'increasing'}]
 * @returns {Object} analysis {valid: true/false, failed: { key: [number] }, data: { webRtcKeyToAnalyze: { valid: true/false, reports: [ { value: number, valid: true/false, difference: number } ] } } }
 * @public
 */
StatsAnalyzer.analyze = function (series) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { analysisKeys: _constants.ANALYSIS_STATS.DEFAULT_KEYS };

  if (!(0, _isArray3.default)(series) || !series.length || !options || !(0, _isArray3.default)(options.analysisKeys) || !options.analysisKeys.length) {
    throw new _parameter2.default('analyzer->analyze#series must be defined as a nonempty array of WebRTCData objects, and options.analysisKeys must be a nonempty array of strings, representing the properties to analyze.');
  }
  var properties = new _set2.default(options.analysisKeys);
  var analysis = { valid: true, failed: {}, data: {} };

  properties.forEach(function (config) {
    var property = config.key;

    analysis.data[property] = { valid: true, reports: [] };
    analysis.failed[property] = [];
    var previous = { value: 0 };
    var index = 0;

    var _loop = function _loop(i) {
      var singular = {};

      (0, _forEach3.default)(series[i].data.getData()[config.prop], function (webrtcData) {
        // eslint-disable-line
        var value = webrtcData[property];

        if (!value || !(0, _isFinite3.default)(value)) {
          return;
        }
        singular.value = value;
        singular.difference = 0;
        singular.valid = false;
        singular.index = index;
        singular.difference = singular.value - previous.value;
        if (config.check === _constants.ANALYSIS_CHECKS.INCREASING && singular.difference > 0) {
          singular.valid = true;
        } else if (config.check === _constants.ANALYSIS_CHECKS.DECREASING && singular.difference < 0) {
          singular.valid = true;
        } else if (config.check === _constants.ANALYSIS_CHECKS.CONSTANT) {
          singular.valid = true;
        } else {
          singular.valid = false;
        }
        if (!singular.valid) {
          analysis.data[property].valid = false;
          analysis.valid = false;
          analysis.failed[property].push(i);
        }
        previous = singular;
        analysis.data[property].reports.push(singular);
      });
      index += 1;
    };

    for (var i = series.length - 1; i > 0; i -= 1) {
      _loop(i);
    }
    if (!analysis.data[property].valid) {
      analysis.valid = false;
    }
  });

  return analysis;
};

exports.default = StatsAnalyzer;
//# sourceMappingURL=analyzer.js.map
